package data.scripts.world.systems;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.PlanetAPI;
import com.fs.starfarer.api.campaign.SectorEntityToken;
import com.fs.starfarer.api.campaign.econ.EconomyAPI;
import com.fs.starfarer.api.campaign.econ.MarketAPI;
import java.util.ArrayList;


public class SystemGenUtils 
{
    public static float getOrbitalPeriod(float primaryRadius, float orbitRadius, float density)
    {
        primaryRadius *= 0.01;
        orbitRadius *= 0.01;

        float mass = (float)Math.floor(4f / 3f * Math.PI * Math.pow(primaryRadius, 3));
        mass *= density;
        float radiusCubed = (float)Math.pow(orbitRadius, 3);
        float period = (float)(2 * Math.PI * Math.sqrt(radiusCubed/mass) * 2);
        return period;
    }

    public static float getDensity(SectorEntityToken primary)
    {
        if (primary instanceof PlanetAPI)
        {
                PlanetAPI planet = (PlanetAPI)primary;
                if (planet.getTypeId().equals("star_dark")) return 8;
                else if (planet.isStar()) return 1;
        }
        return 2;
    }
    
    // by Debido
    // see http://fractalsoftworks.com/forum/index.php?topic=8581.0
    public static void addMarketplace(String factionID, SectorEntityToken primaryEntity, ArrayList<SectorEntityToken> connectedEntities, String name, 
            int size, ArrayList<String> marketConditions, ArrayList<String> submarkets, float tariff) {
        
        EconomyAPI globalEconomy = Global.getSector().getEconomy();
        String planetID = primaryEntity.getId();

        //generate the market ID
        String marketID = planetID + "_market";

        //generate the market
        MarketAPI newMarket = Global.getFactory().createMarket(marketID, name, size);

        //set the faction associated with the market
        newMarket.setFactionId(factionID);

        //set the primary entity related to the market
        newMarket.setPrimaryEntity(primaryEntity);

        //set the base smuggling value (starting value)
        newMarket.setBaseSmugglingStabilityValue(0);

        //set the starting tarrif, could also make this value an input
        newMarket.getTariff().modifyFlat("generator", tariff);

        //add each sub-market types to the mark
        if (null != submarkets) {
            for (String market : submarkets) {
                newMarket.addSubmarket(market);
            }
        }

        //add each market conditions
        for (String condition : marketConditions) {
            newMarket.addCondition(condition);
        }

        //add all connected entities to this marketplace, moons/stations etc.
        if (null != connectedEntities) {
            for (SectorEntityToken entity : connectedEntities) {
                newMarket.getConnectedEntities().add(entity);
            }
        }

        //add the market to the global market place
        globalEconomy.addMarket(newMarket);

        //get the primary entity and associate it back to the market that we've created
        primaryEntity.setMarket(newMarket);

        //to prevent certain issues, make sure the faction is also set for the primary entity
        primaryEntity.setFaction(factionID);

        //get all associated entities and associate it back to the market we've created
        if (null != connectedEntities) {
            for (SectorEntityToken entity : connectedEntities) {
                entity.setMarket(newMarket);
                entity.setFaction(factionID);
            }
        }
    }
}